import swarm.Globals;
import swarm.Selector;
import swarm.defobj.Zone;

import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.objectbase.EmptyProbeMap;
import swarm.objectbase.EmptyProbeMapImpl;

import swarm.activity.ActionGroupImpl;
import swarm.activity.ScheduleImpl;
import swarm.activity.Activity;

import swarm.space.Grid2dImpl;
import swarm.collections.ListImpl;

public class ModelSwarm extends SwarmImpl
{
    public int worldXSize = 80;
    public int worldYSize = 80;

    public AssignValue assignValue;
    public Agent1 a1bug;
    public Agent2 a2bug;

    public Grid2dImpl bugSpace;
    public ScheduleImpl modelSchedule;

    public EmptyProbeMapImpl probeMap;

    public ModelSwarm(Zone aZone)
    {
		super(aZone);

		EmptyProbeMapImpl probeMap = new EmptyProbeMapImpl(aZone, getClass());

		probeMap.addProbe(getProbeForVariable("worldXSize"));
		probeMap.addProbe(getProbeForVariable("worldYSize"));

		Globals.env.probeLibrary.setProbeMap$For(probeMap, getClass());
    }

    public Object buildObjects()
    {
		int xPos=40, yPos=0;

		super.buildObjects();

		assignValue = new AssignValue(Globals.env.globalZone, worldXSize, worldYSize);
		assignValue.allocateValue();

		bugSpace = new Grid2dImpl(Globals.env.globalZone, worldXSize, worldYSize);
		bugSpace.fastFillWithObject(null);

		a1bug = new Agent1(Globals.env.globalZone, assignValue, xPos, yPos, 1);
		a2bug = new Agent2(Globals.env.globalZone, assignValue, xPos, yPos+80, 2);

		return this;
    }

    public Object buildActions()
    {
		Selector sel;
		ActionGroupImpl modelActions;

		super.buildActions();

		modelActions = new ActionGroupImpl(getZone());
		try
	    {
	    	sel = new Selector(a1bug.getClass(), "randomWalkAgent1", false);
	    	modelActions.createActionTo$message(a1bug, sel);
	    }
	    catch (Exception e)
		{
			System.err.println("Exception randomWalkAgent1: " + e.getMessage ());
			System.exit(1);
		}

		try
	    {
	    	sel = new Selector(a1bug.getClass(), "reportPositionAgent1", false);
	    	modelActions.createActionTo$message(a1bug, sel);
	    }
	    catch (Exception e)
		{
			System.err.println("Exception reportPositionAgent1: " + e.getMessage ());
			System.exit(1);
		}

		try
	    {
	    	sel = new Selector(a2bug.getClass(), "randomWalkAgent2", false);
	    	modelActions.createActionTo$message(a2bug, sel);
	    }
	    catch (Exception e)
		{
			System.err.println("Exception randomWalkAgent2: " + e.getMessage ());
			System.exit(1);
		}

		try
	    {
	    	sel = new Selector(a2bug.getClass(), "reportPositionAgent2", false);
	    	modelActions.createActionTo$message(a2bug, sel);
	    }
	    catch (Exception e)
		{
			System.err.println("Exception reportPositionAgent2: " + e.getMessage ());
			System.exit(1);
		}

		modelSchedule = new ScheduleImpl(getZone(), 1);

		modelSchedule.at$createAction(0, modelActions);

		return this;
    }

    public Activity activateIn(Swarm swarmContext)
    {
		super.activateIn(swarmContext);

		modelSchedule.activateIn(this);

		return getActivity();
    }

    public Grid2dImpl getWorld()
    {
		return bugSpace;
    }

    public AssignValue getAssignValue()
    {
		return assignValue;
    }
}