import swarm.Globals;
import swarm.Selector;

import swarm.defobj.Zone;
import swarm.defobj.ZoneImpl;
import swarm.defobj.Symbol;

import swarm.gui.Colormap;
import swarm.gui.ColormapImpl;
import swarm.gui.ZoomRaster;
import swarm.gui.ZoomRasterImpl;

import swarm.space.Value2dDisplay;
import swarm.space.Value2dDisplayImpl;
import swarm.space.Object2dDisplay;
import swarm.space.Object2dDisplay;
import swarm.space.Object2dDisplayImpl;

import swarm.simtoolsgui.GUISwarm;
import swarm.simtoolsgui.GUISwarmImpl;

import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;
import swarm.activity.Activity;

import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.objectbase.EmptyProbeMap;
import swarm.objectbase.EmptyProbeMapImpl;

import swarm.collections.ListImpl;

public class ObserverSwarm extends GUISwarmImpl
{

    public int displayFrequency = 1;
    public int zoomFactor = 8;
    public int simulatedTime = 0;
    public boolean simulationFinished = false;

    Agent1 a1bug;
    Agent2 a2bug;
    ModelSwarm modelSwarm;
    ZoomRaster worldRaster;
    Object2dDisplay bugDisplay1;
    Object2dDisplay bugDisplay2;
    ScheduleImpl displaySchedule;

    public EmptyProbeMapImpl probeMap;

    public ObserverSwarm(Zone aZone)
    {
		super(aZone);

		EmptyProbeMapImpl probeMap = new EmptyProbeMapImpl(aZone, getClass());

		probeMap.addProbe(getProbeForVariable("displayFrequency"));
		probeMap.addProbe(getProbeForVariable("zoomFactor"));

		Globals.env.probeLibrary.setProbeMap$For(probeMap, getClass());
    }

    public Object buildObjects()
    {
		Zone modelZone;
		Colormap colormap;
		Selector sel;

		super.buildObjects();

        modelZone = new ZoneImpl(getZone());
		modelSwarm = (ModelSwarm)Globals.env.lispAppArchiver.getWithZone$key(modelZone, "modelSwarm");

		getControlPanel().setStateStopped();

 		modelSwarm.buildObjects();

		colormap = new ColormapImpl(getZone());
		colormap.setColor$ToName((byte)0, "black");
		colormap.setColor$ToName((byte)1, "red");
		colormap.setColor$ToName((byte)2, "Blue");

   		a1bug.setBugColor((byte)1);
   		a2bug.setBugColor((byte)2);

		worldRaster = new ZoomRasterImpl (getZone(), "worldRaster");
		sel = SwarmUtils.getSelector(this, "_worldRasterDeath_");
		worldRaster.enableDestroyNotification$notificationMethod(this, sel);
		worldRaster.setColormap (colormap);
		worldRaster.setZoomFactor (zoomFactor);
		worldRaster.setWidth$Height((modelSwarm.getWorld()).getSizeX(),(modelSwarm.getWorld()).getSizeY());
		worldRaster.setWindowTitle ("2D Agent World");

		// This instructs the raster to digest all the information we
		// have just given it and to initialize itself for display.
		worldRaster.pack();

		sel = SwarmUtils.getSelector("Agent1", "drawSelfOn");
		sel = SwarmUtils.getSelector("Agent2", "drawSelfOn");

		bugDisplay1 = new Object2dDisplayImpl(getZone(), worldRaster, modelSwarm.getWorld(), sel);
		bugDisplay2 = new Object2dDisplayImpl(getZone(), worldRaster, modelSwarm.getWorld(), sel);

       	return this;
    }

    public Object buildActions()
    {
		Selector sel;
		ActionGroupImpl displayActions;

		super.buildActions();

		modelSwarm.buildActions();

		displayActions = new ActionGroupImpl(getZone());

		sel = SwarmUtils.getSelector(bugDisplay1, "display");
		displayActions.createActionTo$message(bugDisplay1, sel);

		sel = SwarmUtils.getSelector(bugDisplay2, "display");
		displayActions.createActionTo$message(bugDisplay2, sel);

		sel = SwarmUtils.getSelector(worldRaster, "drawSelf");
		displayActions.createActionTo$message(worldRaster, sel);

		sel = SwarmUtils.getSelector(getActionCache(), "doTkEvents");
		displayActions.createActionTo$message(getActionCache(), sel);

		sel = SwarmUtils.getSelector(this, "checkForDone");
		displayActions.createActionTo$message(this, sel);

		displaySchedule = new ScheduleImpl(getZone(), displayFrequency);
		displaySchedule.at$createAction(0, displayActions);

		return this;
    }

    public Activity activateIn(Swarm swarmContext)
    {
		super.activateIn(swarmContext);

		modelSwarm.activateIn(this);

		displaySchedule.activateIn(this);

		return getActivity();
    }

    public void checkForDone()
    {
		if (simulationFinished)
	    {
	    	System.out.println("I said to QUIT!");
	    	modelSwarm.getActivity().terminate();
	    	modelSwarm.drop();
	    	getControlPanel().setStateQuit();
	    }
    }

    public Object _worldRasterDeath_ (Object caller)
    {
		worldRaster.drop ();
		worldRaster = null;
		return this;
    }
}
